#' Loadpage UI module for data selection and upload UI.
#'
#' This function sets up the loadpage UI where it consists of several,
#' options for users to select and upload files.
#'
#' @param id namespace prefix for the module
#'
#' @return This function returns nothing, as it sets up the loadpage UI
#'
#' @export
#' @examples
#' NA
#' 
loadpageUI <- function(id) {
  # fix p tag tooltip
  # fix custom statsshiny tooltip
  ns <- NS(id)
  tagList(
    fluidPage(
      useShinyjs(),
      headerPanel(list("Upload data")),
      p("To run the MSstats Pipeline, please upload your dataset. The required files\
    depend on the spectral processing tool used. Generally the raw data and an \
    annotation file are needed. The output of this step is your experimental \
    data processed in MSstats format. For examples on how to prepare your input \
    please review the MSstats ", a("User Guide", href="https://msstats.org/wp-content/uploads/2020/02/MSstats_v3.18.1_manual_2020Feb26-v2.pdf",
                                   target="_blank")),
      p("PTM data must be processed using MaxQuant or preformatted into MSstats format. For information on how \
  to format your data please see the MSstatsPTM ",
        a("documentation", href="https://www.bioconductor.org/packages/release/bioc/vignettes/MSstatsPTM/inst/doc/MSstatsPTM_LabelFree_Workflow.html",
          target="_blank")),
      p("Note all files must in csv format (unless otherwise indicated) and under 250 MB if using msstatsshiny.com (there is no limit when running the application locally)."),
      p("Some users may have trouble uploading files while using the application via Google Chrome. If the 'Browse...' button does not work please try a different web browser."),
      tags$br(),
      conditionalPanel(
        condition = "input['loadpage-filetype'] == 'sample' && input['loadpage-DDA_DIA'] == 'DDA'",
        p("The sample dataset for DDA acquisition is on its way ")
      ),
      conditionalPanel(
        condition = "input['loadpage-filetype'] && input['loadpage-DDA_DIA'] == 'DIA'",
        p("The sample dataset for DIA acquisition is taken from the publication ",
          a("Selevsek, N. et al. Reproducible and Consistent Quantification of the Saccharomyces Cerevisiae Proteome by SWATH-Mass Spectrometry. Molecular & Cellular Proteomics\u202f: MCP 14.3 (2015): 739\u2013749. ", href = "http://www.mcponline.org/content/14/3/739.long", target="_blank"))
      ),
      conditionalPanel(
        condition = "input['loadpage-filetype'] == 'sample' && input['loadpage-DDA_DIA'] == 'SRM_PRM'",
        p("The sample dataset for SRM/PRM acquisition is taken from the publication ",
          a("Picotti, P. et al. Full dynamic range proteome analysis of S. cerevisiae by targeted proteomics. Cell (2009), 138, 795\u2013806.", href = "http://www.cell.com/cell/fulltext/S0092-8674(09)00715-6", target="_blank"))
      ),
      tags$br(),
      sidebarPanel(
        tags$head(
          tags$style(HTML('#loadpage-proceed1{background-color:orange}')),
          tags$style(HTML('#loadpage-reset1{background-color:orange}')),
          tags$link(rel = "stylesheet", type = "text/css", href = "assets/style.css"),
        ),

        # selection for DIA DDA or SRM/PRM

        radioButtons(ns("BIO"),
                     label <- h4("1. Biological Question",class = "icon-wrapper",icon("question-circle", lib = "font-awesome" ),
                                 div("Select the biological question of interest.", class = "icon-tooltip")),
                     c("Protein"="Protein", "Peptide"="Peptide","PTM"="PTM")
                     ),
        radioButtons(ns("DDA_DIA"),
                     label <- h4("2. Label Type",class = "icon-wrapper",icon("question-circle", lib = "font-awesome" ),
                                 div("Label-free will process all label-free acquisitions including DDA/DIA/SRM/PRM.", class = "icon-tooltip")),
                     c("Label-Free"="LType", "TMT"="TMT")
        ),
        # upload

        radioButtons(ns("filetype"),
                     label <- h4("3. Type of File",class = "icon-wrapper",icon("question-circle", lib = "font-awesome"),
                                 div("Choose the spectral processing tool used to process your data", class = "icon-tooltip")),
                     choices = c("Example dataset" = "sample",
                                 "MSstats Format" = "msstats",
                                 "Skyline" = "sky", "MaxQuant" = "maxq",
                                 "Progenesis" = "prog", "Proteome Discoverer" = "PD",
                                 "OpenMS" = "openms", "Spectronaut" = "spec",
                                 "OpenSWATH" = "open", "DIA-Umpire" = "ump",
                                 "SpectroMine" = "spmin", "FragPipe" = "phil", "DIANN"="diann"),
                     selected = character(0)),
        
        tags$hr(),
        conditionalPanel( condition="input['loadpage-BIO'] != 'PTM' && input['loadpage-filetype'] == 'sample' && input['loadpage-DDA_DIA'] == 'LType' ", 
                     radioButtons(ns("LabelFreeType"),
                     label <- h4("4. Type of Label-Free type",class = "icon-wrapper",icon("question-circle", lib = "font-awesome"),
                                 div("Choose the spectral processing tool used to process your data", class = "icon-tooltip")),
                     choices = c("DDA" = "DDA",
                                 "DIA" ="DIA",
                                 "SRM/PRM" ="SRM_PRM"),
                     selected = character(0))),
        
        tags$hr(),
        conditionalPanel(
          condition = "(input['loadpage-filetype'] =='10col' || input['loadpage-filetype'] =='prog' || input['loadpage-filetype'] =='PD' || input['loadpage-filetype'] =='open'||
                   input['loadpage-filetype'] =='openms' || input['loadpage-filetype'] =='spmin' || input['loadpage-filetype'] == 'phil') && input['loadpage-BIO'] != 'PTM'",
          h4("4. Upload quantification dataset"),
          fileInput(ns('data'), "", multiple = FALSE,
                    accept = NULL),
          radioButtons(ns("sep_data"),
                       label = h5("Column separator in uploaded file", class = "icon-wrapper",icon("question-circle", lib = "font-awesome"),
                                         div("Choose how columns are separated in the uploaded file",class = "icon-tooltip")),
                       c(Comma=",",Semicolon=";", Tab="\t",Pipe="|"),
                       inline = TRUE)
        ),
        conditionalPanel(condition = "input['loadpage-filetype'] == 'msstats' && (input['loadpage-BIO'] != 'PTM' && (input['loadpage-BIO'] != 'PTM' && input['loadpage-DDA_DIA'] != 'TMT'))",
                         h4("4. Upload data in MSstats Format"),
                         fileInput(ns('msstatsdata'), "", multiple = FALSE,
                                   accept = NULL),
                         radioButtons(ns("sep_msstatsdata"),
                                      label = h5("Column separator in uploaded file", class = "icon-wrapper",icon("question-circle", lib = "font-awesome"),
                                                        div("Choose how columns are separated in the uploaded file",class = "icon-tooltip")),
                                      c(Comma=",",Semicolon=";", Tab="\t",Pipe="|"),
                                      inline = TRUE)
        ),
        conditionalPanel(condition = "input['loadpage-filetype'] == 'msstats' && (input['loadpage-BIO'] == 'PTM' || (input['loadpage-BIO'] == 'PTM' && input['loadpage-DDA_DIA'] == 'TMT'))",
                         h4("4. Upload PTM data in MSstats Format"),
                         fileInput(ns('msstatsptmdata'), "", multiple = FALSE,
                                   accept = NULL),
                         radioButtons(ns("sep_msstatsptmdata"),
                                      label = h5("Column separator in uploaded file", class = "icon-wrapper",icon("question-circle", lib = "font-awesome"),
                                                        div("Choose how columns are separated in the uploaded file",class = "icon-tooltip")),
                                      c(Comma=",",Semicolon=";", Tab="\t",Pipe="|"),
                                      inline = TRUE)
        ),
        conditionalPanel(condition = "input['loadpage-filetype'] == 'sky' && input['loadpage-BIO'] != 'PTM'",
                         h4("4. Upload MSstats report from Skyline"),
                         fileInput(ns('skylinedata'), "", multiple = FALSE,
                                   accept = NULL),
                         radioButtons(ns("sep_skylinedata"),
                                      label = h5("Column separator in uploaded file", class = "icon-wrapper",icon("question-circle", lib = "font-awesome"),
                                                        div("Choose how columns are separated in the uploaded file",class = "icon-tooltip")),
                                      c(Comma=",",Semicolon=";", Tab="\t",Pipe="|"),
                                      inline = TRUE)
        ),
        conditionalPanel(condition = "input['loadpage-filetype'] == 'diann' && input['loadpage-BIO'] != 'PTM'",
                         h4("4. Upload MSstats report from DIANN"),
                         fileInput(ns('dianndata'), "", multiple = FALSE,
                                   accept = NULL),
                         radioButtons(ns("sep_dianndata"),
                                      label = h5("Column separator in uploaded file", class = "icon-wrapper",icon("question-circle", lib = "font-awesome"),
                                                 div("Choose how columns are separated in the uploaded file",class = "icon-tooltip")),
                                      c(Comma=",",Semicolon=";", Tab="\t",Pipe="|"),
                                      inline = TRUE)
        ),
        conditionalPanel(condition = "input['loadpage-filetype'] == 'spec' && input['loadpage-BIO'] != 'PTM'",
                         h4("4. Upload MSstats scheme output from Spectronaut"),
                         fileInput(ns('specdata'), "", multiple = FALSE,
                         accept = NULL),
                         radioButtons(ns("sep_specdata"),
                                      label = h5("Column separator in uploaded file", class = "icon-wrapper",icon("question-circle", lib = "font-awesome"),
                                                        div("Choose how columns are separated in the uploaded file",class = "icon-tooltip")),
                                      c(Comma=",",Semicolon=";", Tab="\t",Pipe="|"),
                                      inline = TRUE)
        ),

        conditionalPanel(
          condition = "input['loadpage-filetype'] == 'phil' && input['loadpage-BIO'] == 'PTM'",
          h4("3. Upload PTM msstats dataset"),
          fileInput(ns('ptmdata'), "", multiple = FALSE,
                    accept = NULL),
          h4("4. Upload PTM annotation file"),
          fileInput(ns('annotation'), "", multiple = FALSE,
                    accept = c(".csv")),
          h4("5. Upload global profling msstats dataset (optional)"),
          fileInput(ns('globaldata'), "", multiple = FALSE,
                    accept = NULL),
          h4("6. Upload global profling annotation file (optional)"),
          fileInput(ns('globalannotation'), "", multiple = FALSE,
                    accept = c(".csv")),
          h4("Select the options for pre-processing"),
          textInput(ns("mod_id_col"), h5("Please enter the name of the modification id column", class = "icon-wrapper",icon("question-circle", lib = "font-awesome"),
                                            div("Only part of the string is required. For example if your mod id column is named 'STY:1221.12' you only need to enter 'STY' here.",class = "icon-tooltip")),
                    value = "STY"),
          textInput(ns("localization_cutoff"), h5("Please enter the localization_cutoff", class = "icon-wrapper",icon("question-circle", lib = "font-awesome"),
                                                    div("The probability cutoff used to determine if a modification should be marked or not. If a site cannot be localized it may be dropped depending on the option below.",class = "icon-tooltip")),
                    value = ".75"),
          radioButtons(ns("remove_unlocalized_peptides"), h5("Remove unlocalized peptides", class = "icon-wrapper",icon("question-circle", lib = "font-awesome"),
                                                                div("Should peptides without all sites localized be kept or removed.",class = "icon-tooltip")),
                       c(Yes=TRUE, No=FALSE),
                       inline=TRUE)
        ),
        tags$br(),
        conditionalPanel(
          condition = "(input['loadpage-filetype'] == 'sky' || input['loadpage-filetype'] == 'prog' || input['loadpage-filetype'] == 'PD' || input['loadpage-filetype'] == 'spec' || input['loadpage-filetype'] == 'open'|| input['loadpage-filetype'] =='spmin' || input['loadpage-filetype'] == 'phil' || input['loadpage-filetype'] == 'diann') && input['loadpage-BIO'] != 'PTM'",
          h4("5. Upload annotation File", class = "icon-wrapper",icon("question-circle", lib = "font-awesome"),
                                                 div("Upload manually created annotation file. This file maps MS runs to experiment metadata (i.e. conditions, bioreplicates). Please see Help tab for information on creating this file.",class = "icon-tooltip")),
          fileInput(ns('annot'), "", multiple = FALSE,
                    accept = c(".csv"))
        ),
        tags$br(),
        ## PTM Fragpipe input -- --------------------------------------------------------
        conditionalPanel(condition = "input['loadpage-filetype'] == 'msstats' && (input['loadpage-BIO'] == 'PTM' || (input['loadpage-BIO'] == 'PTM' && input['loadpage-DDA_DIA'] == 'TMT'))",
                         h4("4. (Optional) Upload unmodified data in MSstats Format"),
                         fileInput(ns('unmod'), "", multiple = FALSE,
                                   accept = NULL),
                         tags$br()
                         ),
        conditionalPanel(
          condition = "input['loadpage-filetype'] == 'maxq' && input['loadpage-BIO'] != 'PTM' && (input['loadpage-DDA_DIA'] == 'TMT' || input['loadpage-DDA_DIA'] == 'LType')",
          h4("4. Upload evidence.txt File"),
          fileInput(ns('evidence'), "", multiple = FALSE,
                    accept = NULL),
          h4("5. Upload proteinGroups.txt File"),
          fileInput(ns('pGroup'), "", multiple = FALSE,
                    accept = NULL),
          h4("6. Upload annotation File", class = "icon-wrapper",icon("question-circle", lib = "font-awesome"),
                                                 div("Upload manually created annotation file. This file maps MS runs to experiment metadata (i.e. conditions, bioreplicates). Please see Help tab for information on creating this file.",class = "icon-tooltip")),
          fileInput(ns('annot1'), "", multiple = FALSE,
                    accept = c(".csv"))
        ),
        ## PTM input -- --------------------------------------------------------
        conditionalPanel(
          condition = "(input['loadpage-filetype'] == 'maxq' || input['loadpage-filetype'] == 'PD' || input['loadpage-filetype'] == 'spec' || input['loadpage-filetype'] == 'sky') && (input['loadpage-BIO'] == 'PTM' || (input['loadpage-BIO'] == 'PTM' && input['loadpage-DDA_DIA'] == 'TMT'))",
          h4("4. Upload PTM input.txt File"),
          fileInput(ns('ptm_input'), "", multiple = FALSE,
                    accept = NULL),
          h4("5. Upload annotation File", class = "icon-wrapper",icon("question-circle", lib = "font-awesome"),
                                                 div("Upload manually created annotation file. This file maps MS runs to experiment metadata (i.e. conditions, bioreplicates). Please see Help tab for information on creating this file.",class = "icon-tooltip")),
          fileInput(ns('ptm_annot'), "", multiple = FALSE,
                    accept = c(".csv")),
          h4("6. Upload fasta File", class = "icon-wrapper",icon("question-circle", lib = "font-awesome"),
                                                 div("Upload FASTA file. This file allows us to identify where in the protein sequence a modification occurs.",class = "icon-tooltip")),
          fileInput(ns('fasta'), "", multiple = FALSE),
          h4("7. (Optional) Upload Unmodified Protein input.txt File"),
          fileInput(ns('ptm_protein_input'), "", multiple = FALSE,
                    accept = NULL)
        ),
        conditionalPanel(
          condition = "(input['loadpage-filetype'] == 'maxq') && (input['loadpage-BIO'] == 'PTM' || (input['loadpage-BIO'] == 'PTM' && input['loadpage-DDA_DIA'] == 'TMT'))",
          h4("7. (Optional) Upload Unmodified Protein proteinGroups.txt File"),
          fileInput(ns('ptm_pgroup'), "", multiple = FALSE,
                    accept = NULL)
        ),
        
        conditionalPanel(
          condition = "(input['loadpage-filetype'] == 'maxq') && (input['loadpage-BIO'] == 'PTM' || (input['loadpage-BIO'] == 'PTM' && input['loadpage-DDA_DIA'] == 'TMT'))",
          h4("Modification Label",class = "icon-wrapper",icon("question-circle", lib = "font-awesome"),div("Indicate if experiment was processed using TMT labeling",class = "icon-tooltip")),
          textInput(ns("mod_id_maxq"), "",
                    value="\\(Phospho \\(STY\\)\\)")
        ),
        conditionalPanel(
            condition = "(input['loadpage-filetype'] == 'PD') && (input['loadpage-BIO'] == 'PTM' || (input['loadpage-BIO'] == 'PTM' && input['loadpage-DDA_DIA'] == 'TMT'))",
            h4("Modification Label", class = "icon-wrapper",icon("question-circle", lib = "font-awesome"),div("Indicate if experiment was processed using TMT labeling",class = "icon-tooltip")),
            textInput(ns("mod_id_pd"), "",
                         value="\\(Phospho\\)")
        ),
        conditionalPanel(
          condition = "(input['loadpage-filetype'] == 'spec') && (input['loadpage-BIO'] == 'PTM' || (input['loadpage-BIO'] == 'PTM' && input['loadpage-DDA_DIA'] == 'TMT'))",
          h4("Modification Label",class = "icon-wrapper",icon("question-circle", lib = "font-awesome"),div("Indicate if experiment was processed using TMT labeling",class = "icon-tooltip")),
          textInput(ns("mod_id_spec"),"",
                    value="\\[Phospho \\(STY\\)\\]")
        ),
        conditionalPanel(
          condition = "(input['loadpage-filetype'] == 'maxq' || input['loadpage-filetype'] == 'PD' || input['loadpage-filetype'] == 'spec' || input['loadpage-filetype'] == 'sky') && (input['loadpage-BIO'] == 'PTM' || (input['loadpage-BIO'] == 'PTM' && input['loadpage-DDA_DIA'] == 'TMT'))",
          h4("FASTA file column name",class = "icon-wrapper",icon("question-circle", lib = "font-awesome"),div("Name of column in FASTA file that matches with Protein name column in input. It is critical the values in both columns match so that the modfication can be identified.",class = "icon-tooltip")),
          textInput(ns("mod_id_spec"),"",
                    value="uniprot_iso")
        ),

        ## UMP -----------------------------------------------------------------

        conditionalPanel(
          condition = "input['loadpage-filetype'] == 'ump'",
          h4("5. Upload FragSummary.xls File"),
          fileInput(ns('fragSummary'), "", multiple = FALSE,
                    accept = NULL),
          h4("6. Upload PeptideSummary.xls File"),
          fileInput(ns('peptideSummary'), "", multiple = FALSE,
                    accept = NULL),
          h4("7. Upload ProtSummary.xls File"),
          fileInput(ns('protSummary'), "", multiple = FALSE,
                    accept = NULL),
          h4("8. Upload Annotation File", class = "icon-wrapper",icon("question-circle", lib = "font-awesome"),
                                                 div("Upload manually created annotation file. This file maps MS runs to experiment metadata (i.e. conditions, bioreplicates). Please see Help tab for information on creating this file.",class = "icon-tooltip")),
          fileInput(ns('annot2'), "", multiple = FALSE,
                    accept = c(".csv"))
        ),
        tags$hr(),
        conditionalPanel(condition = "input['loadpage-filetype'] && input['loadpage-DDA_DIA'] == 'TMT' && input['loadpage-filetype'] == 'PD'",
                         h4("Select the options for pre-processing"),
                         textInput(ns("which.proteinid"), h5("Protein Name Column",class = "icon-wrapper",icon("question-circle", lib = "font-awesome"),
                                                                div("Enter the column in your data containing protein names",class = "icon-tooltip")),
                                   value = "Protein.Accessions")),
        conditionalPanel(condition = "input['loadpage-filetype'] && input['loadpage-DDA_DIA'] == 'TMT' && input['loadpage-filetype'] == 'maxq'",
                         h4("Select the options for pre-processing"),
                         textInput(ns("which.proteinid"), h5("Protein Name Column", class = "icon-wrapper",icon("question-circle", lib = "font-awesome"),
                                                                div("Enter the column in your data containing protein names",class = "icon-tooltip")),
                                   value = "Proteins")),

        conditionalPanel(condition = "input['loadpage-filetype'] && input['loadpage-DDA_DIA'] == 'LType' && input['loadpage-filetype'] !== 'sample' && input['loadpage-filetype'] !== 'MRF'",
                         h4("Select the options for pre-processing"),
                         checkboxInput(ns("uniqe_peptides"), "Use unique peptides", value = TRUE),
                         checkboxInput(ns("remove"), "Remove proteins with 1 peptide and charge", value = FALSE)),

        conditionalPanel(condition = "input['loadpage-filetype'] && input['loadpage-DDA_DIA'] == 'LType' && input['loadpage-filetype'] !== 'sample'",
                         checkboxInput(ns("remove"), "Remove proteins with 1 feature", value = FALSE),
                         conditionalPanel(condition = "input['loadpage-filetype'] == 'sky' || input['loadpage-filetype'] == 'spec'|| input['loadpage-filetype'] == 'diann' ",
                                          checkboxInput(ns("q_val"), "Filter with Q-value"),
                                          conditionalPanel(condition = "input['loadpage-q_val']",
                                                           conditionalPanel(condition = "input['loadpage-filetype'] == 'diann'",
                                                                            checkboxInput(ns("MBR"), "MBR Enabled", value = FALSE)
                                                           ),
                                                           numericInput(ns("q_cutoff"), "Q-value cutoff", 0.01, 0, 1, 0.01))),
                         conditionalPanel(condition = "input['loadpage-filetype'] == 'open'",
                                          checkboxInput(ns("m_score"), "Filter with M-score"),
                                          conditionalPanel(condition = "input['loadpage-m_score']",
                                                           numericInput(ns("m_cutoff"), "M-score cutoff", 0.01, 0, 1, 0.01)))
        ),
        disabled(actionButton(inputId = ns("proceed1"), label = "Upload Data"))
      )
      ,
      column(width = 8,
             shinyjs::hidden(uiOutput(ns("summary_tables")))

      )
    )
  )
}
